@@ -3,9 +3,10 @@ import pandas as pd
 import datetime as dt
 import abc
 import scipy.sparse as sp
+from copy import deepcopy
 from typing import Union, List, Tuple, Dict, Sequence
 from eaopack.assets import Node, Asset, Timegrid, convert_time_unit
-from eaopack.optimization import OptimProblem          
+from eaopack.optimization import OptimProblem, SplitOptimProblem
 from eaopack.optimization import Results 
 
 class Portfolio:
@@ -44,7 +45,6 @@ class Portfolio:
         """
         self.timegrid = timegrid
 
-
     def setup_optim_problem(self, prices: dict = None, 
                             timegrid:Timegrid = None, 
                             costs_only:bool = False, 
@@ -129,7 +129,7 @@ class Portfolio:
         if 'disp_factor' not in mapping.columns:
             mapping['disp_factor'] = 1.
         mapping['disp_factor'].fillna(1., inplace = True)
-        mapping['nodal_restr'] = None
+        # mapping['nodal_restr'] = None
 
         def create_nodal_restr(nodes, map_nodes, map_types, map_idx, map_dispf, map_times, timegrid_I, skip_nodes, n_vars):
             """ Specific function creating nodal restrictions """
@@ -138,6 +138,7 @@ class Portfolio:
             cols = np.zeros(0)
             rows = np.zeros(0)
             vals = np.zeros(0)
+            nodal_restr_map_expl = []
             for n in nodes:
                 if (skip_nodes is None) or (not n in skip_nodes):
                     Inode = (map_types=='d') & (map_nodes==n)
@@ -155,8 +156,9 @@ class Portfolio:
                             Itemp[Itemp] = I
                             map_nodal_restr[Itemp] = n_nodal_restr
                             n_nodal_restr +=1
+                            nodal_restr_map_expl.append((t, n))
                             # A = sp.vstack((A, myA))
-            return cols, rows, vals, map_nodal_restr, n_nodal_restr
+            return cols, rows, vals, nodal_restr_map_expl, n_nodal_restr
 
         # # easily readable version -  loop
         # perf = time.perf_counter()
@@ -191,7 +193,7 @@ class Portfolio:
                                                                                 map_times, self.timegrid.I,my_skip_nodes, 
                                                                                 n_vars)
         A = sp.vstack((A, sp.csr_matrix((vals, (rows.astype(np.int64), cols.astype(np.int64))), shape = (n_nodal_restr, n_vars))))
-        mapping['nodal_restr'] = map_nodal_restr.astype(np.int64)
+        # mapping['nodal_restr'] = map_nodal_restr.astype(np.int64)
         ### end cryptic version
 
         b = np.hstack((b,np.zeros(n_nodal_restr))) # must add to zero
@@ -215,7 +217,76 @@ class Portfolio:
             I = mapping['time_step'].isin(timegrid.I[fix_time_window['I']])
             l[I] = fix_time_window['x'][I]
             u[I] = fix_time_window['x'][I]
-        return OptimProblem(c = c, l = l, u = u, A = A, b = b, cType = cType, mapping = mapping)
+        return OptimProblem(c = c, l = l, u = u, A = A, b = b, cType = cType, mapping = mapping, map_nodal_restr = map_nodal_restr)
+
+    def setup_split_optim_problem(self, prices: dict = None,
+                                  timegrid: Timegrid = None,
+                                  interval_size: str = 'd',
+                                  skip_nodes: list = [],
+                                  fix_time_window: Dict = None):
+        """ Set up a split optimization problem for portfolio, i.e. split the timegrid into intervals of size
+            interval_size and create a separate optimization problem for each interval
+
+        Args:
+            prices (dict): Dictionary of price arrays needed by assets in portfolio. Defaults to None
+            timegrid (Timegrid, optional): Discretization grid for portfolio and all assets within.
+                                           Defaults to None, in which case it must have been set previously
+            interval_size (bool): Interval size according to pandas notation ('15min', 'h', 'd', ...). Defaults to 'd'
+            skip_nodes (List): Nodes to be skipped in nodal restrictions (defaults to [])
+            fix_time_window (Dict): Fix results for given indices on time grid to given values. Defaults to None
+                           fix_time_window['I']: Indices on timegrid or alternatively date (all dates before date taken)
+                           fix_time_window['x']: Results.x that results are to be fixed to in time window(full array, all times)
+
+            Returns:
+                SplitOptimProblem: A Split Optimization problem
+        """
+        interval_timepoints = pd.date_range(start=timegrid.start, end=timegrid.end, freq=interval_size,
+                                            tz=timegrid.tz)
+        interval_timepoints = interval_timepoints.append(pd.to_datetime([timegrid.end]))
+        if interval_timepoints[0] != pd.Timestamp(timegrid.start):
+            interval_timepoints = interval_timepoints.insert(0, timegrid.start)
+        prices = timegrid.prices_to_grid(prices)
+        ops = []
+        mappings = []
+        t = 0
+        len_res = 0
+        for i in range(len(interval_timepoints) - 1):
+            start_tmp = interval_timepoints[i]
+            end_tmp = interval_timepoints[i + 1]
+            timegrid_tmp = Timegrid(start_tmp, end_tmp, timegrid.freq, ref_timegrid=timegrid)
+            if timegrid_tmp.T == 0: continue
+            tmp_I = timegrid_tmp.I # the original time steps
+            timegrid_tmp.I = np.array(range(0, timegrid_tmp.T))  
+            # timegrid_tmp.Dt = np.cumsum(timegrid_tmp.dt)           ### use the Dt from the reference --> for discounting
+            prices_tmp = timegrid_tmp.prices_to_grid(prices)
+            op_tmp = self.setup_optim_problem(prices_tmp, timegrid_tmp, skip_nodes=skip_nodes,
+                                              fix_time_window=fix_time_window)
+            mapping_tmp = deepcopy(op_tmp.mapping)
+            # mapping_tmp["time_step"] += t
+            # write original time step IDs to mapping
+            orig_I = [tmp_I[a] for a in mapping_tmp["time_step"]]
+            mapping_tmp["time_step"] = orig_I.copy()
+            if i > 0:
+                # mapping_tmp["nodal_restr"] += mappings[-1]["nodal_restr"].max()
+                for asset in self.assets:
+                    shift = mappings[-1][mappings[-1]["asset"] == asset.name]["index_assets"].max() + 1
+                    mapping_tmp.loc[mapping_tmp["asset"] == asset.name, "index_assets"] += shift
+            mapping_tmp.index += len_res
+            mappings.append(mapping_tmp)
+            # adjust numbering for time steps in map_nodal_restr
+            tmp_nod_rest = []
+            for mynr in op_tmp.map_nodal_restr:
+                tmp_nod_rest.append((tmp_I[mynr[0]], mynr[1]))
+            op_tmp.map_nodal_restr = tmp_nod_rest
+            ops.append(op_tmp)
+            len_res += op_tmp.c.shape[0]
+            t += timegrid_tmp.T
+        mapping = pd.concat(mappings)
+        op = SplitOptimProblem(ops, mapping)
+        self.set_timegrid(timegrid)
+        for a in self.assets:
+            a.set_timegrid(timegrid)
+        return op
 
     def create_cost_samples(self, price_samples: List, timegrid: Timegrid = None) -> List:
         """ create costs vectors for LP on basis of price samples
